--[[
	gMerchant
	Author: Lars Norberg
	
	License:
		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program(see GPL.txt); if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

	Note:
		This AddOn's source code is specifically designed to work with
		World of Warcraft's interpreted AddOn system.
		You have an implicit licence to use this AddOn with these facilities
		since that is its designated purpose as per:
		http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
]]--

-- localization 
local L = gLib:FetchLocale("gMerchant", GetLocale());

-- default settings
local defaults = {
	autorepair 			= true;
	autosell 			= true;
	guildrepair 		= false;
	detailedreport 		= true;
};

-- set up the slash commands
local slash = {
	[1] = {
		name 		= "menu";
		func 		= function() InterfaceOptionsFrame_OpenToCategory(gMerchant.optionspanel.name) end;
		slashes 	= {"gmerchant", "gmerc"};
	};
};

gMerchant_DB = gMerchant_DB or {}

gMerchant = gLib:New({
	name 			= "gMerchant";
	events 			= {
		"ADDON_LOADED";
		"MERCHANT_SHOW";
	};
	slash 			= slash;
	userDB 			= gMerchant_DB;
	defaultDB 		= defaults;
})

local _G = _G;
local format, max, min, print = format, max, min, print;

-- event handlers
function gMerchant:OnInit()
	self:ArgumentCheck( gMerchant_DB, defaults );
	self.optionspanel = self:CreateOptionsPanel({
		title = L["TITLE"];
		version = self.VERSION;
		description = "";
		panel = ((IsAddOnLoaded("gUI") and gUI.optionspanel) or nil);
		option = {
			[0] = {
				name = L["GUI_VENDOR_AUTOSELL"];
				type = "button";
				value = gMerchant_DB.autosell;
				panelfunction = function() gMerchant_DB.autosell = not gMerchant_DB.autosell; end;
			};
			[5] = {
				name = L["GUI_VENDOR_DETAILEDREPORT"];
				type = "button";
				value = gMerchant_DB.detailedreport;
				panelfunction = function() gMerchant_DB.detailedreport = not gMerchant_DB.detailedreport; end;
			};
			[10] = {
				name = L["GUI_VENDOR_AUTOREPAIR"];
				type = "button";
				value = gMerchant_DB.autorepair;
				panelfunction = function() gMerchant_DB.autorepair = not gMerchant_DB.autorepair; end;
			};
			[20] = {
				name = L["GUI_VENDOR_GUILDREPAIR"];
				type = "button";
				value = gMerchant_DB.guildrepair;
				panelfunction = function() gMerchant_DB.guildrepair = not gMerchant_DB.guildrepair; end;
			};
		};
	}, gMerchant_DB, self.defaults);
end

-- Automatically sell poor quality items, and report how much they sell for. 
function gMerchant:MERCHANT_SHOW()

	local currencyChange 		= 0;
	local gotRepaired 			= false;
	local itemValue 			= 0;
	local merchantCanRepair 	= CanMerchantRepair();
	local playerFunds 			= GetMoney();
	local repairCost 			= GetRepairAllCost();
	local sellValue 			= 0;
	local sellstart 			= 0;
	local stackValue 			= 0;
	local useGuildFunds 		= IsInGuild() and gMerchant_DB.guildrepair;
	local usedGuildFunds 		= false;
	local yourGuildFunds 		= min(GetGuildBankWithdrawMoney(),GetGuildBankMoney());
	
	-- iterate our bags for Poor Quality items
	if gMerchant_DB.autosell == true then
		for bag = 0,4,1 do 
			for slot = 1, GetContainerNumSlots(bag), 1 do 
			
				-- first check if there's an actual item in the given slot
				local itemID = GetContainerItemID(bag, slot);

				--  we got an item!
				if itemID then
					-- how many items do we have in this slot?
					local itemCount = (select(2,GetContainerItemInfo(bag, slot)));
					
					-- get the link, rariy and sellprice of our item
					local _, itemLink, itemRarity, _, _, _, _, _, _, _, itemSellPrice = GetItemInfo(itemID); 

					-- if this item is a Poor Quality item, sell it!
					if (itemRarity == 0) then
						itemSellPrice 	= itemSellPrice or 0;			-- make sure no zero-value items screw this up
						itemCount 		= itemCount or 1; 				-- yet another fail safe
						stackValue 		= itemSellPrice * itemCount; 	-- the value of our entire stack
						sellValue 		= sellValue + stackValue; 		-- add it to the total value of all we're selling
						
						if gMerchant_DB.detailedreport then
							-- tell the user what we're selling, and what it's worth
							print(L["MESSAGE_AUTOSELLING"]:format(itemLink, itemCount, string.tag("[money:"..stackValue.."")));
						end
						
						-- sell it!
						UseContainerItem(bag,slot);
					end
				end; 
			end; 
		end
		
		-- add the value of what we sold to the total change in our currency
		currencyChange = currencyChange + sellValue;
	end

	-- report the total profit
	if (sellValue > 0) then print(L["MESSAGE_SELLVALUE"]:format(string.tag("[money:"..sellValue.."]"))); end

	-- Automatically repair if you have sufficient funds when visiting a repair merchant. 
	--	Use Guild Funds if available and chosen in the settings
	if gMerchant_DB.autorepair then
	
		-- is our gear in need of repair?
		if (merchantCanRepair and repairCost>0) then
		
			-- do we have sufficient funds to repair?
			if (max(playerFunds,yourGuildFunds) > repairCost) then
			
				-- can we repair with Guild bank money?
				if (useGuildFunds and (yourGuildFunds > repairCost)) and ((MerchantGuildBankRepairButton:IsEnabled()) and MerchantGuildBankRepairButton:IsShown()) then
					RepairAllItems(1); 
					usedGuildFunds = true;
					gotRepaired = true;
					print(L["MESSAGE_REPAIRCOST_USING_GUILD"]:format(string.tag("[money:"..repairCost.."]")));
		
				-- do we have enough funds with us?
				elseif (playerFunds > repairCost) then
					RepairAllItems(); 
					gotRepaired = true;
					print(L["MESSAGE_REPAIRCOST"]:format(string.tag("[money:"..repairCost.."]")));
					
					-- add the cost to the change in our currency; we're paying for this
					currencyChange = currencyChange - repairCost;
				end
			else
				-- We're poor, and can't repair
				print(L["MESSAGE_NOT_ENOUGH_TO_REPAIR"]);
			end
		end
	end

	-- Decide whether you gained or lost currency from the visiting the Merchant
	if currencyChange > 0 then
		print(L["MESSAGE_VENDORPROFIT"]:format(string.tag("[money:"..currencyChange.."]"))); 
	elseif currencyChange < 0 then 
		print(L["MESSAGE_VENDOREXPENSE"]:format(string.tag("[money:"..(-currencyChange).."]")));
	end
end

